@@ -239,11 +239,12 @@ impl Widget for &mut LegendWidget {
         legend_ui
             .scope(|ui| {
                 let background_frame = Frame {
-                    margin: vec2(8.0, 4.0).into(),
+                    inner_margin: vec2(8.0, 4.0).into(),
                     rounding: ui.style().visuals.window_rounding,
                     shadow: epaint::Shadow::default(),
                     fill: ui.style().visuals.extreme_bg_color,
                     stroke: ui.style().visuals.window_stroke(),
+                    ..Default::default()
                 }
                 .multiply_with_opacity(config.background_alpha);
                 background_frame
